@@ -1,8 +1,11 @@
 from __future__ import annotations
 
+import abc
+import ast
 import collections
 import copy
 import fnmatch
+import gzip
 import io
 import logging
 import pickle
@@ -13,16 +16,16 @@ import weakref
 from defusedxml import minidom
 
 from hippolyzer.lib.base import serialization as se, llsd
+from hippolyzer.lib.base.message.message import Message
 from hippolyzer.lib.base.datatypes import TaggedUnion, UUID, TupleCoord
 from hippolyzer.lib.base.helpers import bytes_escape
 from hippolyzer.lib.base.message.message_formatting import HumanMessageSerializer
 from hippolyzer.lib.proxy.message_filter import MetaFieldSpecifier, compile_filter, BaseFilterNode, MessageFilterNode, \
     EnumFieldSpecifier
-from hippolyzer.lib.proxy.region import CapType
+from hippolyzer.lib.proxy.http_flow import HippoHTTPFlow
+from hippolyzer.lib.proxy.caps import CapType, SerializedCapData
 
 if typing.TYPE_CHECKING:
-    from hippolyzer.lib.proxy.http_flow import HippoHTTPFlow
-    from hippolyzer.lib.base.message.message import Message
     from hippolyzer.lib.proxy.region import ProxiedRegion
     from hippolyzer.lib.proxy.sessions import Session
 
@@ -30,24 +33,42 @@ LOG = logging.getLogger(__name__)
 
 
 class BaseMessageLogger:
+    paused: bool
+
     def log_lludp_message(self, session: Session, region: ProxiedRegion, message: Message):
-        pass
+        if self.paused:
+            return False
+        return self.add_log_entry(LLUDPMessageLogEntry(message, region, session))
 
     def log_http_response(self, flow: HippoHTTPFlow):
-        pass
+        if self.paused:
+            return False
+        # These are huge, let's not log them for now.
+        if flow.cap_data and flow.cap_data.asset_server_cap:
+            return False
+        return self.add_log_entry(HTTPMessageLogEntry(flow))
 
     def log_eq_event(self, session: Session, region: ProxiedRegion, event: dict):
+        if self.paused:
+            return False
+        return self.add_log_entry(EQMessageLogEntry(event, region, session))
+
+    @abc.abstractmethod
+    def add_log_entry(self, entry: AbstractMessageLogEntry):
         pass
 
 
 class FilteringMessageLogger(BaseMessageLogger):
-    def __init__(self):
+    def __init__(self, maxlen=2000):
         BaseMessageLogger.__init__(self)
-        self._raw_entries = collections.deque(maxlen=2000)
+        self._raw_entries = collections.deque(maxlen=maxlen)
         self._filtered_entries: typing.List[AbstractMessageLogEntry] = []
-        self._paused = False
+        self.paused = False
         self.filter: BaseFilterNode = compile_filter("")
 
+    def __iter__(self) -> typing.Iterator[AbstractMessageLogEntry]:
+        return iter(self._filtered_entries)
+
     def set_filter(self, filter_str: str):
         self.filter = compile_filter(filter_str)
         self._begin_reset()
@@ -61,25 +82,7 @@ class FilteringMessageLogger(BaseMessageLogger):
         self._end_reset()
 
     def set_paused(self, paused: bool):
-        self._paused = paused
-
-    def log_lludp_message(self, session: Session, region: ProxiedRegion, message: Message):
-        if self._paused:
-            return
-        self._add_log_entry(LLUDPMessageLogEntry(message, region, session))
-
-    def log_http_response(self, flow: HippoHTTPFlow):
-        if self._paused:
-            return
-        # These are huge, let's not log them for now.
-        if flow.cap_data and flow.cap_data.asset_server_cap:
-            return
-        self._add_log_entry(HTTPMessageLogEntry(flow))
-
-    def log_eq_event(self, session: Session, region: ProxiedRegion, event: dict):
-        if self._paused:
-            return
-        self._add_log_entry(EQMessageLogEntry(event, region, session))
+        self.paused = paused
 
     # Hooks that Qt models will want to implement
     def _begin_insert(self, insert_idx: int):
@@ -94,25 +97,21 @@ class FilteringMessageLogger(BaseMessageLogger):
     def _end_reset(self):
         pass
 
-    def _add_log_entry(self, entry: AbstractMessageLogEntry):
+    def add_log_entry(self, entry: AbstractMessageLogEntry):
         try:
             # Paused, throw it away.
-            if self._paused:
-                return
+            if self.paused:
+                return False
             self._raw_entries.append(entry)
             if self.filter.match(entry):
                 next_idx = len(self._filtered_entries)
                 self._begin_insert(next_idx)
                 self._filtered_entries.append(entry)
                 self._end_insert()
-
-                entry.cache_summary()
-            # In the common case we don't need to keep around the serialization
-            # caches anymore. If the filter changes, the caches will be repopulated
-            # as necessary.
-            entry.freeze()
+                return True
         except Exception:
             LOG.exception("Failed to filter queued message")
+        return False
 
     def clear(self):
         self._begin_reset()
@@ -121,7 +120,27 @@ class FilteringMessageLogger(BaseMessageLogger):
         self._end_reset()
 
 
-class AbstractMessageLogEntry:
+class WrappingMessageLogger(BaseMessageLogger):
+    def __init__(self):
+        self.loggers: typing.List[BaseMessageLogger] = []
+
+    @property
+    def paused(self):
+        return all(x.paused for x in self.loggers)
+
+    def add_log_entry(self, entry: AbstractMessageLogEntry):
+        logged = False
+        for logger in self.loggers:
+            if logger.add_log_entry(entry):
+                logged = True
+        # At least one logger ended up keeping the message around, so let's
+        # cache the summary before we freeze the message.
+        if logged:
+            entry.cache_summary()
+        entry.freeze()
+
+
+class AbstractMessageLogEntry(abc.ABC):
     region: typing.Optional[ProxiedRegion]
     session: typing.Optional[Session]
     name: str
@@ -129,7 +148,7 @@ class AbstractMessageLogEntry:
 
     __slots__ = ["_region", "_session", "_region_name", "_agent_id", "_summary", "meta"]
 
-    def __init__(self, region, session):
+    def __init__(self, region: typing.Optional[ProxiedRegion], session: typing.Optional[Session]):
         if region and not isinstance(region, weakref.ReferenceType):
             region = weakref.ref(region)
         if session and not isinstance(session, weakref.ReferenceType):
@@ -159,6 +178,45 @@ class AbstractMessageLogEntry:
             "SelectedFull": self._current_selected_full(),
         }
 
+    def to_dict(self) -> dict:
+        meta = self.meta.copy()
+
+        def _dehydrate_meta_uuid(key: str):
+            if meta[key]:
+                meta[key] = str(meta[key])
+
+        _dehydrate_meta_uuid("AgentID")
+        _dehydrate_meta_uuid("SelectedFull")
+        _dehydrate_meta_uuid("SessionID")
+
+        return {
+            "type": self.type,
+            "region_name": self.region_name,
+            "agent_id": str(self.agent_id) if self.agent_id is not None else None,
+            "summary": self.summary,
+            "meta": meta,
+        }
+
+    @classmethod
+    @abc.abstractmethod
+    def from_dict(cls, val: dict):
+        pass
+
+    def apply_dict(self, val: dict) -> None:
+        self._region_name = val['region_name']
+        self._agent_id = UUID(val['agent_id']) if val['agent_id'] else None
+        self._summary = val['summary']
+        meta = val['meta'].copy()
+
+        def _hydrate_meta_uuid(key: str):
+            if meta[key]:
+                meta[key] = UUID(meta[key])
+
+        _hydrate_meta_uuid("AgentID")
+        _hydrate_meta_uuid("SelectedFull")
+        _hydrate_meta_uuid("SessionID")
+        self.meta.update(meta)
+
     def freeze(self):
         pass
 
@@ -483,6 +541,26 @@ class HTTPMessageLogEntry(AbstractMessageLogEntry):
             return "application/xml"
         return content_type
 
+    def to_dict(self):
+        val = super().to_dict()
+        val['flow'] = self.flow.get_state()
+        cap_data = val['flow'].get('metadata', {}).get('cap_data_ser')
+        if cap_data is not None:
+            # Have to convert this from a namedtuple to a dict to make
+            # it importable
+            cap_dict = cap_data._asdict()  # noqa
+            val['flow']['metadata']['cap_data_ser'] = cap_dict
+        return val
+
+    @classmethod
+    def from_dict(cls, val: dict):
+        cap_data = val['flow'].get('metadata', {}).get('cap_data_ser')
+        if cap_data:
+            val['flow']['metadata']['cap_data_ser'] = SerializedCapData(**cap_data)
+        ev = cls(HippoHTTPFlow.from_state(val['flow'], None))
+        ev.apply_dict(val)
+        return ev
+
 
 class EQMessageLogEntry(AbstractMessageLogEntry):
     __slots__ = ["event"]
@@ -510,6 +588,17 @@ class EQMessageLogEntry(AbstractMessageLogEntry):
         self._summary = llsd.format_notation(self.event["body"]).decode("utf8")[:500]
         return self._summary
 
+    def to_dict(self) -> dict:
+        val = super().to_dict()
+        val['event'] = llsd.format_notation(self.event)
+        return val
+
+    @classmethod
+    def from_dict(cls, val: dict):
+        ev = cls(llsd.parse_notation(val['event']), None, None)
+        ev.apply_dict(val)
+        return ev
+
 
 class LLUDPMessageLogEntry(AbstractMessageLogEntry):
     __slots__ = ["_message", "_name", "_direction", "_frozen_message", "_seq", "_deserializer"]
@@ -642,3 +731,30 @@ class LLUDPMessageLogEntry(AbstractMessageLogEntry):
         if self._message:
             self._seq = self._message.packet_id
         return self._seq
+
+    def to_dict(self):
+        val = super().to_dict()
+        val['message'] = llsd.format_notation(self.message.to_dict(extended=True))
+        return val
+
+    @classmethod
+    def from_dict(cls, val: dict):
+        ev = cls(Message.from_dict(llsd.parse_notation(val['message'])), None, None)
+        ev.apply_dict(val)
+        return ev
+
+
+def export_log_entries(entries: typing.Iterable[AbstractMessageLogEntry]) -> bytes:
+    return gzip.compress(repr([e.to_dict() for e in entries]).encode("utf8"))
+
+
+_TYPE_CLASSES = {
+    "HTTP": HTTPMessageLogEntry,
+    "LLUDP": LLUDPMessageLogEntry,
+    "EQ": EQMessageLogEntry,
+}
+
+
+def import_log_entries(data: bytes) -> typing.List[AbstractMessageLogEntry]:
+    entries = ast.literal_eval(gzip.decompress(data).decode("utf8"))
+    return [_TYPE_CLASSES[e['type']].from_dict(e) for e in entries]
